#!/bin/bash
# mkimgndx.sh - Make Image HTML Index Page.
# ©2023 J. S. Gilstrap - All Rights Reserved.
# GNU General Public License ≥2.1 - https://www.gnu.org/license
bv=$(bash --version | grep "^GNU.*bash.*version") ; bv=${bv#*version }
[ "${bv%%.*}" -lt 4 ] && { echo "This script requires Bash version 4 or greater. Quitting!" ; exit 1 ; }
# Editor
ED=vi

# Your Name.
yourName="Your Name"
# Set desired pixel image width.
imgWdth=480
# Output file name.
outFile=imgndx.html
# Page Title
Title="My Pictures"
# Favicon.
icon=favicon.png
# Keywords.
keys="photos,pictures,images"
# Copyright stamp.
copy="&copy;2023"
# Font stack.
fnt="'Lucida Handwriting','QK Marisa','Comic Sans MS',BookmanJFPro-Italic,'Arial Rounded MT Bold','Trebuchet MS',Arial,Calibri,Verdana,Geneva,Candara,Helvetica,Tahoma,Sans"
# DB Separator.
sep=":"

# Set variables, test for magick, source config.db.
gm="" ; calc=1 ; cnt=0 ; pWdth=$((${imgWdth}+16))
which gm > /dev/null 2>&1 && gm=gm || which identify > /dev/null 2>&1 || calc=0
[ -r config.db -a -s config.db ] && source config.db

# Get image size and resize to imgWdth if different to specify in image tag for browser scaling.
getSize()
	{
	local h s w
	[ $calc -eq 0 ] && { size=" width=\"${imgWdth}\"" ; return ; }
	s=$($gm identify "${1/// }") ; s=${s%+*+*} ; s=${s##* } ; w=${s%x*} ; h=${s#*x}
	[ "$w" -ne "$imgWdth" ] 2> /dev/null && { h=$(((1000*${h}*${imgWdth}/${w}+500)/1000)) ; w=$imgWdth ; }
	[ -n "$w" -a -n "$h" ] && size=" width=\"${w}\" height=\"${h}\"" || size=" width=\"${imgWdth}\""
	}

# Check for image.db. If not then create one.
if [ ! -f image.db ]
then
	printf "Do you wish to create an image.db from images in current directory? [y|Y] : "
	read yn

	if [ "${yn^}" = Y ]
	then
		db=""
		while read img
		do
			cmt=${img//_/ } ; cmt=${cmt%.webp} ; cmt=${cmt%.gif}
			cmt=${cmt%.jpg} ; cmt=${cmt%.png}  ; cmt=${cmt%.svg}
			cmt=${cmt//+/ &amp; } ; db="${db}${img}${sep}${cmt}\n"
		done < <(ls *.gif *.jpg *.png *.svg *.webp 2> /dev/null)
		printf "$db" > image.db ; unset ls db img cmt
	else
		echo "No Image Database. Quitting!"
		exit 1
	fi

	yn=""
	printf "Edit database? [y|Y] : "
	read yn

	if [ "${yn^}" = Y ]
	then ed=""
		printf "Using editor [1m${ED^^}[0m. Specify another? : "
		read ed
		[ -n "$ed" ] && ED="$ed"
		$ED image.db
	fi
fi

[ -f "$outFile" ] && mv "$outFile" "${outFile}-bak" 
# Open file handle.
exec 3> $outFile

# Create html header.
if [ -r html.head -a -s html.head ]
then
	hh="$(< html.head)"
	hh="${hh//\#keys\#/$keys}" ; hh="${hh//\#fnt\#/$fnt}" ; hh="${hh//\#Title\#/$Title}"
	hh="${hh//\#pWdth\#/$pWdth}" ; hh="${hh//\#imgWdth\#/$imgWdth}"
	hh="${hh//\#yourName\#/$yourName}" ; hh="${hh//\#icon\#/$icon}"
	echo "$hh" >&3
else
  IFS=$' \n'

while read line ; do echo "$line" ; done << EoF >&3
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional/EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
	<title>${Title}</title>
	<link rel="shortcut icon" type="image/png" href="${icon}"> 
	<meta http-equiv="Content-Type" content="text/html"; charset="utf-8">
	<meta name="author" content="${yourName}">
	<meta name="keywords" content="${keys}">
	<meta name="viewport" content="width=device-width, initial-scale=0.64, maxium-scale=1.5, minimum-scale=0.5"> 
	<style>
		   a {text-decoration:none; }
		body { width:${pWdth}px; }
	</style>
</head>
<body text="#6000c0" bgcolor="#fef7f2" link="#4040ff" alink="#ff0029" vlink="#8600e6">

<center>
<font face="${fnt}">
<font size="7">
<b>${Title}</b>
</font>
<br>
<font size="5"><b>
<br>

EoF
  IFS=$' \t\n' 
fi

# Create image tags.
while read line
do
	((cnt++))
	img=${line%$sep*}
	cmt=${line#*$sep}
	getSize ${img// /}
	printf "<a id=\"${cnt}\"><img src=\"${img}\" alt=\"${cmt}\" title=\"${cmt}\"${size} loading=\"lazy\"><br>#${cnt}<br><br>\n" >&3
	echo "$img"
done < image.db

printf "\n${cnt} Images\n\n"

# Create html footer.
if [ -r html.foot -a -s html.foot ]
then
	hf="$(< html.foot)" ; hf="${hf//\#yourName\#/$yourName}"
	hf="${hf//\#copy\#/$copy}" ; echo "$hf" >&3
else
  IFS=$' \n'

while read line ; do echo "$line" ; done << EoF >&3
</b></font>
<br>
<font size="4"><b>
${copy} ${yourName}<br>All Rights Reserved.<br>
</b></font>
<br>
</font>
</center>
</body>
</html>
EoF

fi

# Close file handle.
exec 3<&-
